//	Sudoku Widget v1.0.0
//	Copyright 2005 - Andrea Arevalo
//	www.AndreaArevalo.com

// Constants
var CELLSIZE = 34;
var CELLSPACE = 1;
var BLOCKSIZE = CELLSIZE * 3 + CELLSPACE * 4;
var BLOCKSPACE = 3;
var BOARDSIZE = BLOCKSIZE * 3 + BLOCKSPACE * 4;

// Controls
var txtPen = new Array();
var txtPencil = new Array();
var imgCell = new Array();
var imgBlock = new Array();
var imgBoard = new Image();
var imgNumber = new Array();
var imgCellMode = new Image();
var imgMsgBox = new Image();

// Global variables
var gsGameId = "";
var gsDifficultyLevel = "";
var gsPuzzle = new Array();
var gsSolution = new Array();
var gsValidMoves = new Array();
var gsCorrectMoves = new Array();
var gsCellMode = new Array();
var gsPenMark = new Array();
var gsPencilMark = new Array();
var giActiveCellIndex = 0;

function initWidget() {

	var i, j, iCol, iRow, iBlock, iBlockCol, iBlockRow;

	suppressUpdates();

	// Draw board background
	imgBoard.src     = "Resources/Images/board.png";
	imgBoard.width   = BLOCKSIZE * 3 + BLOCKSPACE * 4;
	imgBoard.height  = BLOCKSIZE * 3 + BLOCKSPACE * 4;
	imgBoard.hOffset = 6;
	imgBoard.vOffset = 45;

	// Draw cell blocks
	for (i = 0; i < 9; i++) {
		iRow = Math.floor(i/3);
		iCol = Math.floor(i - 3 * iRow);
		imgBlock[i] = new Image();
		imgBlock[i].src = "Resources/Images/block.png";
		imgBlock[i].width = BLOCKSIZE;
		imgBlock[i].height = BLOCKSIZE;
		imgBlock[i].hOffset = imgBoard.hOffset + BLOCKSPACE + iCol * (BLOCKSIZE+BLOCKSPACE);
		imgBlock[i].vOffset = imgBoard.vOffset + BLOCKSPACE + iRow * (BLOCKSIZE+BLOCKSPACE);
	}

	// Draw cells
	for (i = 0; i < 81; i++) {
		iRow = Math.floor(i/9);
		iCol = Math.floor(i - 9 * iRow);
		iBlock = Math.floor(iCol/3)+3*Math.floor(iRow/3);
		iBlockCol = iCol - 3*Math.floor(iCol/3);
		iBlockRow = iRow - 3*Math.floor(iRow/3);

		imgCell[i] = new Image();
		imgCell[i].src = "Resources/Images/cell.png";
		imgCell[i].width = CELLSIZE;
		imgCell[i].height = CELLSIZE;
		imgCell[i].hOffset = imgBlock[iBlock].hOffset + CELLSPACE + iBlockCol * (CELLSIZE+CELLSPACE);
		imgCell[i].vOffset = imgBlock[iBlock].vOffset + CELLSPACE + iBlockRow * (CELLSIZE+CELLSPACE);
		imgCell[i].onMouseDown = "selectCell("+i+");";

		txtPen[i] = new TextArea();
		txtPen[i].data =  "";
		txtPen[i].color =  "#FFFFFF";
		txtPen[i].width = CELLSIZE-5;
		txtPen[i].height = CELLSIZE;
		txtPen[i].font = "Arial";
		txtPen[i].size = 24;
		txtPen[i].scrollbar = false;
		txtPen[i].editable = false;
		txtPen[i].hOffset = imgCell[i].hOffset + 10;
		txtPen[i].vOffset = imgCell[i].vOffset + 5;

		txtPencil[i] = new TextArea();
		txtPencil[i].data =  "";
		txtPencil[i].color =  "#FFFFFF";
		txtPencil[i].width = CELLSIZE-5;
		txtPencil[i].height = CELLSIZE;
		txtPencil[i].font = "Arial";
		txtPencil[i].size = 10;
		txtPencil[i].scrollbar = false;
		txtPencil[i].editable = false;
		txtPencil[i].hOffset = imgCell[i].hOffset + 5;
		txtPencil[i].vOffset = imgCell[i].vOffset + 1;
		txtPencil[i].opacity = 0;
	}


	// Cell mode toggle
	imgCellMode.src="Resources/Images/imgLock.png";
	imgCellMode.width = 40;
	imgCellMode.height = 40;
	imgCellMode.hOffset = 10;
	imgCellMode.vOffset = 380;
	imgCellMode.onMouseUp = "processKeyStroke(' ','Space');"


	// Numeric pad
	for (i = 0; i < 10; i++) {
		imgNumber[i] = new Image();
		imgNumber[i].src = "Resources/Images/imgNumber"+i+".png";
		imgNumber[i].width = 20;
		imgNumber[i].height = 20;
		imgNumber[i].hOffset = 60+i*27;
		imgNumber[i].vOffset = 390;	
		imgNumber[i].onMouseUp = "processKeyStroke('"+ i + "','0x3"+ i + "');"
	}
	imgNumber[0].onMouseUp = "processKeyStroke('', 'Backspace');"

	// Message box
	imgMsgBox.opacity = 0;
	imgMsgBox.src="Resources/Images/imgPuzzleSolved.png";
	imgMsgBox.width = 322;
	imgMsgBox.height = 40;
	imgMsgBox.hOffset = 10;
	imgMsgBox.vOffset = 380;

	resumeUpdates();

}

function initGame(bLoadNewBoard) {

	var sGamePack, iGameNumber, iGamePackNumber, sGameData, sDigits, i, n;

	if (bLoadNewBoard) {
		sDigits = "0123456789Aa";
		gsDifficultyLevel = preferences.difficultylevel.value;
		imgDifficultyLevel.src = "resources/images/imgDifficulty" + gsDifficultyLevel + ".png";
		iGamePackNumber = 1 + Math.floor(Math.random()*2);
		sGamePack = filesystem.readFile("Resources/GamePacks/GamePack "+ iGamePackNumber + "-" + gsDifficultyLevel + ".sgp", true);	
		iGameNumber = Math.floor(Math.random()*sGamePack.length);
		sGameData = sGamePack[iGameNumber];
		gsGameId = "" + iGamePackNumber + "-" + gsDifficultyLevel + "-" + iGameNumber;
		for (i = 0; i < 81; i++) {
			if ((sGameData.charAt(i) >= "A") && (sGameData.charAt(i) <= "I")) {
				n = sGameData.charCodeAt(i) - sDigits.charCodeAt(10);
				gsSolution[i] = sDigits.charAt(n+1);
				gsPuzzle[i] = "*";
			} else {
				n = sGameData.charCodeAt(i) - sDigits.charCodeAt(11);
				gsSolution[i] = sDigits.charAt(n+1);
				gsPuzzle[i] = ".";
			}
		}
	}

	for (i = 0; i < 81; i++) {
		if (gsPuzzle[i] == "*") {
			gsCellMode[i] = "*";
			gsPenMark[i] = gsSolution[i];
		} else if (preferences.defaultcellmode.value=="pen") {
			gsCellMode[i] = ".";
			gsPenMark[i] = " ";
		} else {
			gsCellMode[i] = "-";
			gsPenMark[i] = " ";
		}
		gsPencilMark[i] = "         ";
		gsValidMoves[i] = " ";
		gsCorrectMoves[i] = " ";
		showCellValue(i);
		setCellBackground(i);
	}
	selectCell(0);
	hideMsgBox();
}

function applyPreferences() {

	var i;

	checkGameAids();
	for (i = 0; i < 81; i++) {
		setCellBackground(i);
	}
}

function showMsgBox() {

	if (imgMsgBox.opacity == 0) {
		imgMsgBox.opacity = 255;
		for (i = 0; i < 81; i++) {
			imgCell[i].opacity = 0;
			txtPen[i].color = "#FFFFFF";
		}
		imgCellMode.opacity=0;
		for (i = 0; i < 10; i++) {
			imgNumber[i].opacity = 0;
		}
	}		
}

function hideMsgBox() {

	if (imgMsgBox.opacity != 0) {
		imgMsgBox.opacity = 0;
		for (i = 0; i < 81; i++) {
			imgCell[i].opacity = 255;
		}
		imgCellMode.opacity=255;
		for (i = 0; i < 10; i++) {
			imgNumber[i].opacity = 255;
		}
	}
}

function displaySolution() {

	var i, j, k, l, iCellIndex, sHTML;

	sHTML = "";
	sHTML = sHTML + "<HTML><HEAD><TITLE>Sudoku</TITLE></HEAD><BODY>";
	sHTML = sHTML + "<img src=\"sudoku.gif\" width=200 height=30><br><br>"
	sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=4 bgcolor=\"#000000\">";
	for (i = 0; i < 3; i++) {
		sHTML = sHTML + "<tr>";
		for (j = 0; j < 3; j++) {
			sHTML = sHTML + "<td>";
			sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=1 bgcolor=\"#808080\">";
			for (k = 0; k < 3; k++) {				
				sHTML = sHTML + "<tr>";
				for (l = 0; l < 3; l++) {
					iCellIndex = 9*(i*3+k)+(j*3+l);
					sHTML = sHTML + "<td><img src=\""+gsSolution[iCellIndex]+".gif\" width=25 height=25></td>";
				}
				sHTML = sHTML + "</tr>";	
			}
			sHTML = sHTML + "</table>";
			sHTML = sHTML + "</td>";
		}
		sHTML = sHTML + "</tr>";
	}
	sHTML = sHTML + "</table>";
	sHTML = sHTML + "<img src=\"difficulty"+gsDifficultyLevel+".gif\" width=200 height=30>"
	sHTML = sHTML + "<br><br>";
	sHTML = sHTML + "<img src=\"puzzleno.gif\" width=90 height=25>"
	l = gsGameId.length;
	for (i = 0; i < l; i++) {
		if (gsGameId.charAt(i) == "-") {
			sHTML = sHTML + "<img src=\"dash.gif\" width=10 height=25>"			
		} else {
			sHTML = sHTML + "<img src=\""+ gsGameId.charAt(i) +".gif\" width=25 height=25>"			
		}
	}
	sHTML = sHTML + "</BODY></HTML>";
	filesystem.writeFile("Resources/PrintPreview/sudoku.htm", sHTML);
	filesystem.open("Resources/PrintPreview/sudoku.htm");
}

function printPuzzle() {

	var i, j, k, l, iCellIndex, sHTML;

	sHTML = "";
	sHTML = sHTML + "<HTML><HEAD><TITLE>Sudoku</TITLE></HEAD><BODY>";
	sHTML = sHTML + "<img src=\"sudoku.gif\" width=200 height=30><br><br>"
	sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=4 bgcolor=\"#000000\">";
	for (i = 0; i < 3; i++) {
		sHTML = sHTML + "<tr>";
		for (j = 0; j < 3; j++) {
			sHTML = sHTML + "<td>";
			sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=1 bgcolor=\"#808080\">";
			for (k = 0; k < 3; k++) {				
				sHTML = sHTML + "<tr>";
				for (l = 0; l < 3; l++) {
					iCellIndex = 9*(i*3+k)+(j*3+l);
					if (gsPuzzle[iCellIndex] == "*") {
						sHTML = sHTML + "<td><img src=\""+gsSolution[iCellIndex]+".gif\" width=25 height=25></td>";
					} else {
						sHTML = sHTML + "<td><img src=\"0.gif\" width=25 height=25></td>";	
					}
				}
				sHTML = sHTML + "</tr>";	
			}
			sHTML = sHTML + "</table>";
			sHTML = sHTML + "</td>";
		}
		sHTML = sHTML + "</tr>";
	}
	sHTML = sHTML + "</table>";
	sHTML = sHTML + "<img src=\"difficulty"+ gsDifficultyLevel + ".gif\" width=200 height=30>"
	sHTML = sHTML + "<br><br><br><br>";
	if (preferences.printmode.value=="puzzleandsolution"){
		sHTML = sHTML + "<img src=\"solution.gif\" width=100 height=15><br>"
		sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=2 bgcolor=\"#808080\">";
		for (i = 0; i < 3; i++) {
			sHTML = sHTML + "<tr>";
			for (j = 0; j < 3; j++) {
				sHTML = sHTML + "<td>";
				sHTML = sHTML + "<table border=0 cellpadding=0 cellspacing=1 bgcolor=\"#808080\">";
				for (k = 0; k < 3; k++) {				
					sHTML = sHTML + "<tr>";
					for (l = 0; l < 3; l++) {
						iCellIndex = 9*(i*3+k)+(j*3+l);
						sHTML = sHTML + "<td><img src=\""+gsSolution[iCellIndex]+"s.gif\" width=15 height=15></td>";
					}
					sHTML = sHTML + "</tr>";	
				}
				sHTML = sHTML + "</table>";
				sHTML = sHTML + "</td>";
			}
			sHTML = sHTML + "</tr>";
		}
		sHTML = sHTML + "</table>";
	}
	sHTML = sHTML + "</BODY></HTML>";
	filesystem.writeFile("Resources/PrintPreview/sudoku.htm", sHTML);
	filesystem.open("Resources/PrintPreview/sudoku.htm");
}

function showCellValue(iCellIndex) {

	var i, s;

	// Cell is locked
	if (gsCellMode[iCellIndex] == "*") {
		txtPencil[iCellIndex].opacity = 0;
		txtPen[iCellIndex].opacity = 255;
		txtPen[iCellIndex].size = 24;
		txtPen[iCellIndex].data = gsPenMark[iCellIndex];
		txtPen[iCellIndex].hOffset = imgCell[iCellIndex].hOffset + 10;
		txtPen[iCellIndex].vOffset = imgCell[iCellIndex].vOffset + 2;
		txtPen[iCellIndex].color = "#999966";

	// Pencil mode
	} else if (gsCellMode[iCellIndex] == "-") {
		txtPen[iCellIndex].opacity = 0;
		txtPencil[iCellIndex].opacity = 255;
		txtPencil[iCellIndex].size = 10;
		txtPencil[iCellIndex].data = gsPencilMark[iCellIndex].replace(/ +/g,"");
		txtPencil[iCellIndex].hOffset = imgCell[iCellIndex].hOffset + 1;
		txtPencil[iCellIndex].vOffset = imgCell[iCellIndex].vOffset + 1;
		txtPencil[iCellIndex].color = "#333300";

	// Pen mode
	} else {
		txtPencil[iCellIndex].opacity = 0;
		txtPen[iCellIndex].opacity = 255;
		txtPen[iCellIndex].size = 24;
		txtPen[iCellIndex].data = gsPenMark[iCellIndex];
		txtPen[iCellIndex].hOffset = imgCell[iCellIndex].hOffset + 10;
		txtPen[iCellIndex].vOffset = imgCell[iCellIndex].vOffset + 2;
		txtPen[iCellIndex].color = "#666633";
	}
}

function selectCell(iCellIndex) {

	var iPrevCellIndex;

	iPrevCellIndex = giActiveCellIndex;
	giActiveCellIndex = iCellIndex;

	setCellBackground(iPrevCellIndex);
	setCellBackground(giActiveCellIndex);

	// Cell is locked
	if (gsCellMode[iCellIndex] == "*") {
		imgCellMode.src="Resources/Images/imgLock.png";
		imgCellMode.tooltip="Cell is locked";

	// Pencil mode
	} else if (gsCellMode[iCellIndex] == "-") {
		imgCellMode.src="Resources/Images/imgPencil.png";
		imgCellMode.tooltip="Pencil mode";

	// Pen mode
	} else {
		imgCellMode.src="Resources/Images/imgPen.png";
		imgCellMode.tooltip="Pen mode";
	}

}

function clearCellValue(iCellIndex) {

	var i;

	// Cell is locked, do nothing
	if (gsCellMode[iCellIndex] == "*") {

	// Pencil mode
	} else if (gsCellMode[iCellIndex] == "-") {
		for (i = 0; i < 10; i++) {
			gsPencilMark[iCellIndex] = "         ";
		}

	// Pen mode
	} else {
		gsPenMark[iCellIndex] = " ";
	}
	showCellValue(iCellIndex);
	checkGameAids();
	setCellBackground(iCellIndex);
}

function setCellValue(iCellIndex, sValue) {

	var n;

	// Cell is locked, do nothing
	if (gsCellMode[iCellIndex] == "*") {

	// Pencil mode (toggle pencil marks)
	} else if (gsCellMode[iCellIndex] == "-") {
		n = parseInt(sValue);
		if (gsPencilMark[iCellIndex].charAt(n-1) == " ") {
			gsPencilMark[iCellIndex] = gsPencilMark[iCellIndex].substr(0,n-1)+sValue+gsPencilMark[iCellIndex].substr(n,9-n);
		} else {
			gsPencilMark[iCellIndex] = gsPencilMark[iCellIndex].substr(0,n-1)+" "+gsPencilMark[iCellIndex].substr(n,9-n);
		}

	// Pen
	} else {
		gsPenMark[iCellIndex] = sValue;
	}
	showCellValue(iCellIndex);
	checkGameAids();
	setCellBackground(iCellIndex);
	checkSolution();
}

function setCellBackground(iCellIndex) {

	var sCellBackground;

	sCellBackground = "Resources/Images/";

	if (iCellIndex == giActiveCellIndex) {
		sCellBackground = sCellBackground + "active";
	}
	if ((preferences.gameaids.value == "highlightincorrectmoves")&&(gsCellMode[iCellIndex]==".")&&(gsCorrectMoves[iCellIndex] == "x")) {
		sCellBackground = sCellBackground + "cellx.png";
	} else if((preferences.gameaids.value == "highlightinvalidmoves")&&(gsCellMode[iCellIndex]==".")&&(gsValidMoves[iCellIndex] == "x")) {
		sCellBackground = sCellBackground + "cellx.png";
	} else {
		sCellBackground = sCellBackground + "cell.png";
	}
	if (imgCell[iCellIndex].src != sCellBackground) {
		imgCell[iCellIndex].src = sCellBackground;
	}
}

function toggleCellMode(iCellIndex) {

	if (iCellIndex < 0)
		iCellIndex = giActiveCellIndex;

	// Cell is locked, do nothing
	if (gsCellMode[iCellIndex] == "*") {

	// Switch from Pencil to Pen
	} else if (gsCellMode[iCellIndex] == "-") {
		gsCellMode[iCellIndex] = ".";
		imgCellMode.src="Resources/Images/imgPen.png";
		imgCellMode.tooltip="Pen mode";
		showCellValue(iCellIndex);
		setCellBackground(iCellIndex);

	// Switch from Pen to Pencil
	} else {
		gsCellMode[iCellIndex] = "-";
		imgCellMode.src="Resources/Images/imgPencil.png";
		imgCellMode.tooltip="Pencil mode";
		showCellValue(iCellIndex);
		setCellBackground(iCellIndex);
	}
	checkGameAids();
	checkSolution();
}

function checkGameAids() {

	var i, j, k, n, iCellIndex, iRow, iCol, iBlock, iBlockCol, iBlockRow, sDigits;

	sDigits = "0123456789";
	iRow = Math.floor(giActiveCellIndex/9);
	iCol = Math.floor(giActiveCellIndex - 9 * iRow);
	iBlock = Math.floor(iCol/3)+3*Math.floor(iRow/3);
	iBlockCol = iCol - 3*Math.floor(iCol/3);
	iBlockRow = iRow - 3*Math.floor(iRow/3);

	// Is move correct? (not blank, not equal to solution)
	iCellIndex = giActiveCellIndex;
	if ((gsPenMark[iCellIndex] != " ")&& (gsPenMark[iCellIndex] != gsSolution[iCellIndex])) {
		gsCorrectMoves[iCellIndex] = "x";
	} else {
		gsCorrectMoves[iCellIndex] = " ";
	}

	// Assume all cells in row, col and block are valid
	for (i=0; i < 9; i++) {
		iCellIndex = iRow * 9 + i;
		gsValidMoves[iCellIndex] = " ";
	}
	for (i=0; i < 9; i++) {
		iCellIndex = i * 9 + iCol;
		gsValidMoves[iCellIndex] = " ";
	}
	for (j = 0; j < 3; j++) {
		for (k = 0; k < 3; k++) {
			iCellIndex = 3*Math.floor(iCol/3) + 27*Math.floor(iRow/3) + j * 9 + k;
			gsValidMoves[iCellIndex] = " ";
		}
	}

	// Validate row, column and block
	for (i = 1; i < 10; i++) {

		// Row
		for (iRow=0; iRow < 9; iRow++) {
			n = 0;
			for (j = 0; j < 9; j++) {
				iCellIndex = iRow * 9 + j;
				if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] != "-")) {
					n++;
				}
			}
			for (j = 0; j < 9; j++) {
				iCellIndex = iRow * 9 + j;
				if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] == ".")&&(n > 1)) {
					gsValidMoves[iCellIndex] = "x";
				}
			}
		}

		// Column
		for (iCol=0; iCol < 9; iCol++) {
			n = 0;
			for (j = 0; j < 9; j++) {
				iCellIndex = j * 9 + iCol;
				if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] != "-")) {
					n++;
				}
			}
			for (j = 0; j < 9; j++) {
				iCellIndex = j * 9 + iCol;
				if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] == ".")&&(n > 1)) {
					gsValidMoves[iCellIndex] = "x";
				}
			}
		}

		// Block
		for (iBlock = 0; iBlock < 9; iBlock++) {
			n = 0;
			for (iRow = 0; iRow < 3; iRow++) {
				for (iCol = 0; iCol < 3; iCol++) {
					iCellIndex = 27 * Math.floor(iBlock/3) + 3 * (iBlock - 3 * Math.floor(iBlock/3)) + 9 * iRow + iCol;
					if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] != "-")) {
						n++;
					}

				}
			}
			for (iRow = 0; iRow < 3; iRow++) {
				for (iCol = 0; iCol < 3; iCol++) {
					iCellIndex = 27 * Math.floor(iBlock/3) + 3 * (iBlock - 3 * Math.floor(iBlock/3)) + 9 * iRow + iCol;
					if ((gsPenMark[iCellIndex] == sDigits.charAt(i))&&(gsCellMode[iCellIndex] == ".")&&(n > 1)) {
						gsValidMoves[iCellIndex] = "x";
					}
				}
			}
		}
	}

	// Refresh display
	if ((preferences.gameaids.value == "highlightinvalidmoves")) {
		for (iCellIndex=0; iCellIndex < 81; iCellIndex++) {
			setCellBackground(iCellIndex);
		}
	}
}

function checkSolution() {

	var i, bSolved;

	bSolved = true;
	i = 0;
	while ((i < 81) && (bSolved)) {
		bSolved = ((gsCellMode[i] != "-") && (gsPenMark[i] == gsSolution[i]));
		i++;
	}

	if (bSolved) {
		showMsgBox();
		if (preferences.puzzlesolvedsound.value != "none") {
			play("Resources/Sounds/"+preferences.puzzlesolvedsound.value+".wav");
		}
	}
}

function processMenu(sCmd) {

	if (sCmd=="new") {
		initGame(true);
	} else if (sCmd=="reset") {
		initGame(false);
	} else if (sCmd=="solve") {
		displaySolution();	
	} else if (sCmd=="print") {
		printPuzzle();	
	} else if (sCmd=="help") {
		openURL("http://www.andreaarevalo.com/widgets.asp?widget=sudoku&version=100&cmd=help");	
	} else if (sCmd=="checkforupdates") {
		openURL('http://www.andreaarevalo.com/widgets.asp?widget=sudoku&version=100&cmd=checkforupdates');
	} else if (sCmd=="reportproblem") {
		openURL('mailto:mail@andreaarevalo.com?subject=Sudoku - Problem Report');
	} else if (sCmd=="suggestenhancement") {
		openURL('mailto:mail@andreaarevalo.com?subject=Sudoku - Enhancement Suggestion');
	}
}

function processKeyStroke(key, keystring) {

	var iRow, iCol, iBlock;

	if (imgMsgBox.opacity != 0) {
		return;
	}

	iRow = Math.floor(Math.floor(giActiveCellIndex/9));
	iCol = Math.floor(giActiveCellIndex - 9 * iRow);
	iBlock = Math.floor(iCol/3)+3*Math.floor(iRow/3);

	if ((keystring >= "0x31") && (keystring <= "0x39")) {
		setCellValue(giActiveCellIndex, key);
	} else if ((keystring == "Backspace")||(keystring == "ForwardDelete")) {
		clearCellValue(giActiveCellIndex);
	} else if ((keystring == "Space")) {
		toggleCellMode(giActiveCellIndex);
	} else if (keystring=="UpArrow") {
		if (iRow == 0) {
			selectCell(iCol+9*8);
		} else {
			selectCell(iCol+9*(iRow-1));
		}
	} else if (keystring=="DownArrow") {
		if (iRow == 8) {
			selectCell(iCol+9*0);
		} else {
			selectCell(iCol+9*(iRow+1));
		}
	} else if (keystring=="LeftArrow") {
		if (iCol == 0) {
			selectCell(8+9*iRow);
		} else {
			selectCell(iCol-1+9*iRow);
		}
	} else if (keystring=="RightArrow") {
		if (iCol == 8) {
			selectCell(0+9*iRow);
		} else {
			selectCell(iCol+1+9*iRow);
		}
	}
}

initWidget();
initGame(true);